/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.transform;

import org.enhydra.zeus.InvalidCollectionTypeException;
import org.enhydra.zeus.util.ClassUtils;

/**
 * <p>
 *  <code>TransformerOptions</code> is used to specify options used in
 *    <code>{@link org.enhydra.zeus.Transformer}</code> and related classes
 *    and processes. It specifies global options for transformation such
 *    as packages, collection types, and more.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class TransformerOptions {
    
    /** The default package for all classes, unless overridden */
    protected String defaultPackage;
    
    /** The default interface package for this transformation */
    protected String interfacePackage;
    
    /** The default implementation package for this transformation */
    protected String implementationPackage;
    
    /** The default collection class type */
    protected int defaultCollectionType;
    
    /**
     * <p>
     *  This default constructor sets up default variable values.
     * </p>
     */
    public TransformerOptions() {
        this("", ClassUtils.COLLECTION_TYPE_LIST);
    }
    
    /**
     * <p>
     *  This convenience constructor allows options to be set on
     *    construction.
     * </p>
     *
     * @param defaultPackage the package for all generated classes.
     * @param defaultCollectionType constant for the collection type to use.
     */
    public TransformerOptions(String defaultPackage, 
                              int defaultCollectionType) {
        this.defaultPackage = defaultPackage;
        interfacePackage = null;
        implementationPackage = null;
        try {
            setDefaultCollectionType(defaultCollectionType);
        } catch (InvalidCollectionTypeException e) {
            try {
                setDefaultCollectionType(ClassUtils.COLLECTION_TYPE_LIST);
            } catch (InvalidCollectionTypeException neverHappens) { }
        }
    }
    
    /**
     * <p>
     *  This convenience constructor allows options to be set on
     *    construction.
     * </p>
     *
     * @param interfacePackage the package for all generated interfaces.
     * @param implementationPackage the package for all generated implementation
     *        packages.
     * @param defaultCollectionType constant for the collection type to use.
     */
    public TransformerOptions(String interfacePackage,
                              String implementationPackage,
                              int defaultCollectionType) {
        defaultPackage = "";
        interfacePackage = interfacePackage;
        implementationPackage = implementationPackage;
        try {
            setDefaultCollectionType(defaultCollectionType);
        } catch (InvalidCollectionTypeException e) {
            try {
                setDefaultCollectionType(ClassUtils.COLLECTION_TYPE_LIST);
            } catch (InvalidCollectionTypeException neverHappens) { }
        }
    }
    
    /**
     * <p>
     *  This sets the default and global package for all generated
     *    classes, unless a specific package for interfaces and/or
     *    implementations is supplied, which then overrides this
     *    setting. Those packages can be set using the
     *    <code>{@link #setInterfacePackage}</code> and
     *    <code>{@link #setImplementationPackage}</code> methods.
     * </p><p>
     *  There is intentionally not a <code>getDefaultPackage()</code> method.
     *    Instead, <code>{@link #getInterfacePackage}</code> and
     *    <code>{@link #getImplementationPackage}</code> should be used,
     *    which return correct values based on the default and specific
     *    packages set for both options.
     *
     * @param interfacePackage the package for generated interfaces.
     */
    public void setDefaultPackage(String defaultPackage) {
        if (defaultPackage == null) {
            throw new IllegalArgumentException("A TransformerOptions cannot " +
                "have a null value for the default package. To use the " +
                "default package, specify an empty String (\"\").");
        }
        
        this.defaultPackage = defaultPackage;
    }
    
    /**
     * <p>
     *  This sets the default and global interface package for generated
     *    classes.
     * </p>
     *
     * @param interfacePackage the package for generated interfaces.
     */
    public void setInterfacePackage(String interfacePackage) {
        if (interfacePackage == null) {
            throw new IllegalArgumentException("A TransformerOptions cannot " +
                "have a null value for the interface package. To use the " +
                "default package, specify an empty String (\"\").");
        }
        this.interfacePackage = interfacePackage;
    }
    
    /**
     * <p>
     *  This returns the default package in which generated interfaces are
     *    placed.
     * </p>
     *
     * @return <code>String</code> - the default interface package.
     */
    public String getInterfacePackage() {
        if (interfacePackage != null) {
            return interfacePackage;
        } else {
            return defaultPackage;
        }
    }
    
    /**
     * <p>
     *  This sets the default and global interface package for generated
     *    classes.
     * </p>
     *
     * @param interfacePackage the package for generated interfaces.
     */
    public void setImplementationPackage(String implementationPackage) {
        if (implementationPackage == null) {
            throw new IllegalArgumentException("A TransformerOptions cannot " +
                "have a null value for the implementation package. To use " +
                "the default package, specify an empty String (\"\").");
        }
        this.implementationPackage = implementationPackage;
    }
    
    /**
     * <p>
     *  This returns the default package in which generated implementations are
     *    placed.
     * </p>
     *
     * @return <code>String</code> - the default implementation package.
     */
    public String getImplementationPackage() {
        if (implementationPackage != null) {
            return implementationPackage;
        } else {
            return defaultPackage;
        }
    }
    
    /**
     * <p>
     *  This allows setting of the default collection type for transformations,
     *    through constants defined in this class. If the supplied constant is 
     *    an illegal type, an 
     *    <code>{@link InvalidCollectionTypeException}</code> is thrown.
     * </p>
     *
     * @param defaultCollectionType the collection type to use in
     *        transformations, as an <code>int</code> constant.
     * @throws <code>InvalidCollectionTypeException</code> - when an illegal
     *         collection type is supplied.
     * @see <code>{@link #COLLECTION_TYPE_LIST}</code>
     * @see <code>{@link #COLLECTION_TYPE_ARRAY}</code>
     */
    public void setDefaultCollectionType(int defaultCollectionType)
        throws InvalidCollectionTypeException {
        
        // Ensure a legal value
        if (!ClassUtils.isCollectionConstant(defaultCollectionType)) {
            throw new InvalidCollectionTypeException();
        }
        
        // If legal, set as default collection type
        this.defaultCollectionType = defaultCollectionType;
    }
    
    /**
     * <p>
     *  This allows setting of the default collection type for transformations,
     *    through supplying a <code>String</code> class name, such as
     *    <code>java.util.List</code>. If the supplied type is an illegal type,
     *    an <code>{@link InvalidCollectionTypeException}</code> is thrown.
     * </p>
     *
     * @param defaultCollectionTypeString the String representation of the
     *        collection type to use.
     * @throws <code>InvalidCollectionTypeException</code> - when an illegal
     *         collection type is supplied.
     */
    public void setDefaultCollectionType(String defaultCollectionTypeString)
        throws InvalidCollectionTypeException {
            
        // If legal, set as default collection type
        this.defaultCollectionType = 
            ClassUtils.getCollectionTypeAsInt(defaultCollectionTypeString);
    }
    
    /**
     * <p>
     *  This will return the default collection class name for this set
     *    of transformation options. This will return a fully-qualified class
     *    name, such as <code>java.util.List</code>.
     * </p>
     *
     * @return <code>String</code> - the class name for the collection type.
     */
    public String getDefaultCollectionType() {
        return ClassUtils.getCollectionTypeAsString(defaultCollectionType);
    }
}
